package ij.io;
import java.io.*;
import java.net.*;

/** ImageJ uses this class loader to load plugins from the plugins
	folder. It's based on the FileClassLoader from
	"Java Class Libraries: Second Edition, Vol. 1"
	(http://java.sun.com/docs/books/chanlee/second_edition/vol1/).
*/
public class PluginClassLoader extends ClassLoader {
    String path;

    public PluginClassLoader(String path) {
        this.path = path;
    }

    // Loads the bytes from file 
    Class loadIt(String classname) throws ClassNotFoundException {
        // To get file name, remove the package name, if any.
        //String filename;
        //if (classname.indexOf(".") >= 0) {
        //    filename = classname.substring(classname.lastIndexOf(".")+1);
        //} else {
        //    filename = classname;
        //}
        
        String filename = classname.replace('.','/');
        
        // Make sure the filename ends with .class
        filename += ".class";

        File fullname = new File(path, filename);
		// ij.IJ.write("class file name: " + fullname);
        try {
            // Read in the byte codes.
            InputStream is = new FileInputStream(fullname);
            int bufsize = (int)fullname.length();
            byte buf[] = new byte[bufsize];
            is.read(buf, 0, bufsize);
            is.close();

            // Define the class
            return defineClass(classname, buf, 0, buf.length);
        } catch (Exception e) {
            throw new ClassNotFoundException(classname);
        }
    }

    protected synchronized Class loadClass(String name, boolean resolve)
            throws ClassNotFoundException {
        // Try to find it from the cache
        Class c = findLoadedClass(name);
        //System.out.println(name + ((c == null) ? " not " : " ") + "in cache");

        // Not in cache
        if (c == null) {
            // See if it can be loaded by system class loader
            try {
                // No need to call resolveClass() on the result
                // because findSystemClass() loads and links the class.
                return findSystemClass(name);
            } catch (ClassNotFoundException e) {
            }
        
            // Try to get it from file
            c = loadIt(name);
        }

        // Link class if asked to do so
        if (c != null && resolve) {
	    //System.out.println("Resolving class: " + name);
            resolveClass(c);
        }
        return c;
    }

}
